@@ -52,7 +52,7 @@ char last_error[ERROR_BUFFER_SIZE];
 
 void version(void)
 {
-	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2011 folkert@vanheusden.com\n");
+	fprintf(stderr, "HTTPing v" VERSION ", (C) 2003-2012 folkert@vanheusden.com\n");
 #ifndef NO_SSL
 	fprintf(stderr, "SSL support included\n");
 #endif
@@ -221,9 +221,9 @@ int main(int argc, char *argv[])
 	char *proxy = NULL, *proxyhost = NULL;
 	int proxyport = 8080;
 	int portnr = 80;
-	char *get = NULL, *request;
-	int req_len;
-	int c;
+	char *get = NULL, *request = NULL;
+	int req_len = 0;
+	int c = 0;
 	int count = -1, curncount = 0;
 	double wait = 1.0;
 	int audible = 0;
@@ -232,18 +232,15 @@ int main(int argc, char *argv[])
 	int timeout=30;
 	char show_statuscodes = 0;
 	char use_ssl = 0;
-	SSL_CTX *client_ctx = NULL;
 	char *ok_str = "200";
 	char *err_str = "-1";
 	char *useragent = NULL;
 	char *referer = NULL;
-	char *host;
+	char *host = NULL;
 	char *pwd = NULL;
 	char *usr = NULL;
 	char *cookie = NULL;
-	int port;
-	struct sockaddr_in6 addr;
-	struct addrinfo ai;
+	int port = 0;
 	char resolve_once = 0;
 	char auth_mode = 0;
 	char have_resolved = 0;
@@ -251,7 +248,7 @@ int main(int argc, char *argv[])
 	int nagios_exit_code = 2;
 	double avg_httping_time = -1.0;
 	int get_instead_of_head = 0;
-	char *buffer;
+	char *buffer = NULL;
 	int page_size = sysconf(_SC_PAGESIZE);
 	char show_Bps = 0, ask_compression = 0;
 	int Bps_min = 1 << 30, Bps_max = 0;
@@ -260,7 +257,6 @@ int main(int argc, char *argv[])
 	char show_bytes_xfer = 0, show_fp = 0;
 	int fd = -1;
 	SSL *ssl_h = NULL;
-	BIO *s_bio;
 	struct sockaddr_in *bind_to = NULL;
 	struct sockaddr_in bind_to_4;
 	struct sockaddr_in6 bind_to_6;
@@ -268,6 +264,7 @@ int main(int argc, char *argv[])
 	char split = 0, use_ipv6 = 0;
 	char persistent_connections = 0, persistent_did_reconnect = 0;
 	char no_cache = 0;
+	char *getcopyorg = NULL;
 
 	static struct option long_options[] =
 	{
@@ -413,7 +410,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'h':
-				hostname = optarg;
+				hostname = strdup(optarg);
 				break;
 
 			case 'p':
@@ -526,6 +523,8 @@ int main(int argc, char *argv[])
 		char *slash, *colon;
 		char *getcopy = mystrdup(get, "get request");
 
+		getcopyorg = getcopy;
+
 		if (strncasecmp(getcopy, "http://", 7) == 0)
 		{
 			getcopy += 7;
@@ -608,6 +607,7 @@ int main(int argc, char *argv[])
 	}
 
 #ifndef NO_SSL
+	SSL_CTX *client_ctx = NULL;
 	if (use_ssl)
 	{
 		client_ctx = initialize_ctx();
@@ -656,7 +656,7 @@ int main(int argc, char *argv[])
 	if (auth_mode) { 
 		char auth_string[255];
 		char b64_auth_string[255];
-		if ( usr == NULL )
+		if (usr == NULL)
 			error_exit("Basic Authnetication (-A) can only be used with a username and/or password (-U -P) ");
 		sprintf(auth_string,"%s:%s",usr,pwd); 
 		enc_b64(auth_string, strlen(auth_string), b64_auth_string);
@@ -685,16 +685,21 @@ int main(int argc, char *argv[])
 	host = proxyhost?proxyhost:hostname;
 	port = proxyhost?proxyport:portnr;
 
+	struct sockaddr_in6 addr;
+	struct addrinfo *ai = NULL, *ai_use;
+
+	double started_at = get_ts();
 	if (resolve_once)
 	{
-		memset(&addr, 0x00, sizeof(addr));
-
-		if (resolve_host(host, &addr, &ai, use_ipv6, port) == -1)
+		if (resolve_host(host, &ai, use_ipv6, port) == -1)
 		{
 			err++;
 			emit_error();
 			have_resolved = 1;
 		}
+
+		ai_use = select_resolved_host(ai, use_ipv6);
+		get_addr(ai_use, &addr);
 	}
 
 	if (persistent_connections)
@@ -726,18 +731,28 @@ persistent_loop:
 			{
 				memset(&addr, 0x00, sizeof(addr));
 
-				if (resolve_host(host, &addr, &ai, use_ipv6, port) == -1)
+				if (ai)
+				{
+					freeaddrinfo(ai);
+
+					ai = NULL;
+				}
+
+				if (resolve_host(host, &ai, use_ipv6, port) == -1)
 				{
 					err++;
 					emit_error();
 					break;
 				}
 
+				ai_use = select_resolved_host(ai, use_ipv6);
+				get_addr(ai_use, &addr);
+
 				have_resolved = 1;
 			}
 
 			if ((persistent_connections && fd < 0) || (!persistent_connections))
-				fd = connect_to(bind_to_valid?bind_to:NULL, &ai, timeout);
+				fd = connect_to((struct sockaddr *)(bind_to_valid?bind_to:NULL), ai, timeout);
 
 			if (fd == -3)	/* ^C pressed */
 				break;
@@ -769,9 +784,8 @@ persistent_loop:
 #ifndef NO_SSL
 				if (use_ssl && ssl_h == NULL)
 				{
-					int rc;
-
-					rc = connect_ssl(fd, client_ctx, &ssl_h, &s_bio, timeout);
+					BIO *s_bio = NULL;
+					int rc = connect_ssl(fd, client_ctx, &ssl_h, &s_bio, timeout);
 					if (rc != 0)
 					{
 						close(fd);
@@ -1096,20 +1110,29 @@ persistent_loop:
 			break;
 		}
 
+		fflush(NULL);
+
 		if (curncount != count && !stop)
 			usleep((useconds_t)(wait * 1000000.0));
 	}
 
-	avg_httping_time = avg / (double)ok;
+	if (ok)
+		avg_httping_time = avg / (double)ok;
+	else
+		avg_httping_time = -1.0;
 
+	double total_took = get_ts() - started_at;
 	if (!quiet && !machine_readable && !nagios_mode)
 	{
 		printf("--- %s ping statistics ---\n", get);
 
+		if (curncount == 0 && err > 0)
+			fprintf(stderr, "internal error! (curncount)\n");
+
 		if (count == -1)
-			printf("%d connects, %d ok, %3.2f%% failed\n", curncount, ok, (((double)err) / ((double)curncount)) * 100.0);
+			printf("%d connects, %d ok, %3.2f%% failed, time %.0fms\n", curncount, ok, (((double)err) / ((double)curncount)) * 100.0, total_took * 1000.0);
 		else
-			printf("%d connects, %d ok, %3.2f%% failed\n", curncount, ok, (((double)err) / ((double)count)) * 100.0);
+			printf("%d connects, %d ok, %3.2f%% failed, time %.0fms\n", curncount, ok, (((double)err) / ((double)count)) * 100.0, total_took * 1000.0);
 
 		if (ok > 0)
 		{
@@ -1153,6 +1176,11 @@ error_exit:
 		return nagios_exit_code;
 	}
 
+	freeaddrinfo(ai);
+	free(request);
+	free(buffer);
+	free(getcopyorg);
+
 	if (ok)
 		return 0;
 	else
